"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseAbstractClient = exports.isUpdateFailureReason = exports.UpdateFailureReason = void 0;
var tslib_1 = require("tslib");
var utils_1 = require("../utils");
var UpdateFailureReason;
(function (UpdateFailureReason) {
    UpdateFailureReason[UpdateFailureReason["AlreadyUpdated"] = 0] = "AlreadyUpdated";
    UpdateFailureReason[UpdateFailureReason["BadRequest"] = 1] = "BadRequest";
    UpdateFailureReason[UpdateFailureReason["RPCError"] = 2] = "RPCError";
})(UpdateFailureReason || (exports.UpdateFailureReason = UpdateFailureReason = {}));
function isUpdateFailureReason(x) {
    return Number.isInteger(x);
}
exports.isUpdateFailureReason = isUpdateFailureReason;
var BaseAbstractClient = (function () {
    function BaseAbstractClient(eventSearchConfig, cachingMechanism) {
        if (eventSearchConfig === void 0) { eventSearchConfig = { fromBlock: 0, maxBlockLookBack: 0 }; }
        this.eventSearchConfig = eventSearchConfig;
        this.cachingMechanism = cachingMechanism;
        this.firstBlockToSearch = 0;
        this.latestBlockSearched = 0;
        this._isUpdated = false;
    }
    Object.defineProperty(BaseAbstractClient.prototype, "isUpdated", {
        get: function () {
            return this._isUpdated;
        },
        set: function (value) {
            if (this._isUpdated === true && !value) {
                throw new Error("Cannot set isUpdated to false once it is true");
            }
            this._isUpdated = value;
        },
        enumerable: false,
        configurable: true
    });
    BaseAbstractClient.prototype.updateSearchConfig = function (provider) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var fromBlock, toBlock, maxBlockLookBack;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        fromBlock = this.firstBlockToSearch;
                        toBlock = this.eventSearchConfig.toBlock;
                        if (!(0, utils_1.isDefined)(toBlock)) return [3, 1];
                        if (fromBlock > toBlock) {
                            throw new Error("Invalid event search config fromBlock (".concat(fromBlock, ") > toBlock (").concat(toBlock, ")"));
                        }
                        return [3, 3];
                    case 1: return [4, provider.getBlockNumber()];
                    case 2:
                        toBlock = _a.sent();
                        if (toBlock < fromBlock) {
                            return [2, UpdateFailureReason.AlreadyUpdated];
                        }
                        _a.label = 3;
                    case 3:
                        maxBlockLookBack = this.eventSearchConfig.maxBlockLookBack;
                        return [2, { fromBlock: fromBlock, toBlock: toBlock, maxBlockLookBack: maxBlockLookBack }];
                }
            });
        });
    };
    BaseAbstractClient.prototype.assertUpdated = function () {
        if (!this.isUpdated) {
            throw new Error("Client not updated");
        }
    };
    BaseAbstractClient.prototype.hasCachingMechanism = function () {
        return (0, utils_1.isDefined)(this.cachingMechanism);
    };
    return BaseAbstractClient;
}());
exports.BaseAbstractClient = BaseAbstractClient;
//# sourceMappingURL=BaseAbstractClient.js.map